@@ -1,3 +1,4 @@
+#include <functional>
 #include "genfile/bgen/View.hpp"
 #include "genfile/bgen/IndexQuery.hpp"
 #include "pgenlib_internal.h"
@@ -9,21 +10,22 @@ using namespace plink2;
 struct BgenXfer {
 	dataPtr dp;
 	std::vector<double> prob;
-	int row;
+	int dr;
 	double total;
 	int nrows;
-	BgenXfer(dataPtr &_dp) : dp(_dp), total(0), nrows(0) {};
+	std::function<bool(int)> skipFn;
+	BgenXfer(dataPtr &_dp, std::function<bool(int)> _skipFn) : dp(_dp), total(0), nrows(0), skipFn(_skipFn) {};
 	void initialise( std::size_t number_of_samples, std::size_t number_of_alleles ) {}
 	void set_min_max_ploidy(genfile::bgen::uint32_t min_ploidy, genfile::bgen::uint32_t max_ploidy,
 				genfile::bgen::uint32_t min_entries, genfile::bgen::uint32_t max_entries)
 	{
-		row = 0;
+		dr = 0;
 		if (min_ploidy != 2 || max_ploidy != 2 || min_entries != 3 || max_entries != 3) {
 			mxThrow("set_min_max_ploidy %u %u %u %u, not implemented",
 				min_ploidy, max_ploidy, min_entries, max_entries);
 		}
 	}
-	bool set_sample( std::size_t i ) { return true; }
+	bool set_sample( std::size_t i ) { return !skipFn(i); }
 	void set_number_of_entries(std::size_t ploidy,
 				   std::size_t number_of_entries,
 				   genfile::OrderType order_type,
@@ -44,7 +46,7 @@ struct BgenXfer {
 				total += dosage;
 				nrows += 1;
 			}
-			dp.realData[row++] = dosage;
+			dp.realData[dr++] = dosage;
 		}
 	}
 
@@ -54,7 +56,7 @@ struct BgenXfer {
 	}
 };
 
-class LoadDataBGENProvider : public LoadDataProvider<LoadDataBGENProvider> {
+struct LoadDataBGENProvider2 : public LoadDataProvider2<LoadDataBGENProvider2> {
 	int cpIndex;
 	genfile::bgen::View::UniquePtr bgenView;
 
@@ -80,13 +82,13 @@ class LoadDataBGENProvider : public LoadDataProvider<LoadDataBGENProvider> {
 	virtual int getNumVariants();
 };
 
-int LoadDataBGENProvider::getNumVariants()
+int LoadDataBGENProvider2::getNumVariants()
 {
 	if (bgenView.get() == 0) return 0;
 	return bgenView->number_of_variants();
 }
 
-void LoadDataBGENProvider::loadRowImpl(int index)
+void LoadDataBGENProvider2::loadRowImpl(int index)
 {
 	// discard m_postheader_data? TODO
 	if (columns.size() != 1) mxThrow("%s: bgen only has 1 column, not %d",
@@ -106,9 +108,9 @@ void LoadDataBGENProvider::loadRowImpl(int index)
 		query->initialise();
 		bgenView->set_query( query ) ;
 		curRecord = index;
-		if (rows != int(bgenView->number_of_samples())) {
+		if (srcRows != int(bgenView->number_of_samples())) {
 			mxThrow("%s: %s has %d rows but %s has %d samples",
-				name, dataName, rows, filePath.c_str(),
+				name, dataName, srcRows, filePath.c_str(),
 				int(bgenView->number_of_samples()));
 		}
 		loadCounter += 1;
@@ -120,13 +122,13 @@ void LoadDataBGENProvider::loadRowImpl(int index)
 	if (!bgenView->read_variant( &SNPID, &rsid, &chromosome, &position, &alleles )) {
 		mxThrow("%s: %s has no more varients", name, filePath.c_str());
 	}
-	BgenXfer xfer(stripeData[0]);
+	BgenXfer xfer(stripeData[0], [&](int rx)->bool{ return skipRow(rx); });
 	bgenView->read_genotype_data_block(xfer);
 	curRecord += 1;
 
 	auto &rc = *rawCols;
 	for (int cx=0; cx < int(columns.size()); ++cx) {
-		rc[ columns[cx] ].ptr = stripeData[cx];
+		rc[ columns[cx] ].setBorrow(stripeData[cx]);
 	}
 	if (checkpoint) {
 		auto &cv = *checkpointValues;
@@ -140,7 +142,7 @@ void LoadDataBGENProvider::loadRowImpl(int index)
 	}
 }
 
-class LoadDataPGENProvider : public LoadDataProvider<LoadDataPGENProvider> {
+struct LoadDataPGENProvider2 : public LoadDataProvider2<LoadDataPGENProvider2> {
 	int cpIndex;
 
 	struct PgenFileInfoDtor {
@@ -163,7 +165,7 @@ class LoadDataPGENProvider : public LoadDataProvider<LoadDataPGENProvider> {
 		}
 	};
 	typedef std::unique_ptr< PgenReaderStruct, PgenReaderStructDtor > PgenReaderStructPtr;
-		
+
 	PgenFileInfoPtr pgen_info;
 	PgenReaderStructPtr pgen_state;
 	uintptr_t* pgen_subset_include_vec;
@@ -172,10 +174,10 @@ class LoadDataPGENProvider : public LoadDataProvider<LoadDataPGENProvider> {
 	uintptr_t* pgen_dosage_present;
 	uint16_t* pgen_dosage_main;
 
+	std::vector<int> srcDestMap;
+
 	virtual const char *getName() { return "pgen"; };
-	virtual void init(SEXP rObj) {
-		requireFile(rObj);
-	}
+	virtual void init(SEXP rObj);
 	virtual void addCheckpointColumns(std::vector< std::string > &cp)
 	{
 		cpIndex = cp.size();
@@ -185,52 +187,88 @@ class LoadDataPGENProvider : public LoadDataProvider<LoadDataPGENProvider> {
 	virtual int getNumVariants();
 };
 
-static const int kGenoToFactor[4] = {1, 2, 3, NA_INTEGER};
+void LoadDataPGENProvider2::init(SEXP rObj)
+{
+	requireFile(rObj);
 
-// TODO: investigate GenoarrLookup16x8bx2()
-static void GenoarrToFactor(const uintptr_t* genoarr, uint32_t sample_ct, int *geno_out) {
-  const uint32_t word_ct_m1 = (sample_ct - 1) / kBitsPerWordD2;
-  int* write_iter = geno_out;
-  uint32_t subgroup_len = kBitsPerWordD2;
-  for (uint32_t widx = 0; ; ++widx) {
-    if (widx >= word_ct_m1) {
-      if (widx > word_ct_m1) {
-        return;
-      }
-      subgroup_len = ModNz(sample_ct, kBitsPerWordD2);
-    }
-    uintptr_t geno_word = genoarr[widx];
-    for (uint32_t uii = 0; uii != subgroup_len; ++uii) {
-      *write_iter++ = kGenoToFactor[geno_word & 3];
-      geno_word >>= 2;
-    }
-  }
+	srcDestMap.resize(srcRows);
+	for (int rx=0, dr=0; rx < srcRows; ++rx) {
+		if (skipRow(rx)) {
+			srcDestMap[rx] = -1;
+		} else {
+			srcDestMap[rx] = dr++;
+		}
+	}
 }
 
-void Dosage16ToDoubles(const double* geno_double_pair_table, const uintptr_t* genoarr, const uintptr_t* dosage_present, const uint16_t* dosage_main, uint32_t sample_ct, uint32_t dosage_ct, double* geno_double) {
-  GenoarrLookup16x8bx2(genoarr, geno_double_pair_table, sample_ct, geno_double);
-  if (dosage_ct) {
-    const uint16_t* dosage_main_iter = dosage_main;
-    uintptr_t sample_uidx_base = 0;
-    uintptr_t cur_bits = dosage_present[0];
-    for (uint32_t dosage_idx = 0; dosage_idx != dosage_ct; ++dosage_idx) {
-      const uintptr_t sample_uidx = BitIter1(dosage_present, &sample_uidx_base, &cur_bits);
-      geno_double[sample_uidx] = S_CAST(double, *dosage_main_iter++) * 0.00006103515625;
+void myGenoarrLookup16x8bx2(const uintptr_t* genoarr, const void* table16x8bx2, uint32_t sample_ct,
+														std::function<bool(int)> skipFn, void* __restrict result)
+{
+  const uint64_t* table_alias = S_CAST(const uint64_t*, table16x8bx2);
+  uint64_t* result_iter = S_CAST(uint64_t*, result);
+	int rx=0, dr=0;
+  const uint32_t sample_ctl2m1 = (sample_ct - 1) / kBitsPerWordD2;
+  uint32_t loop_len = kBitsPerWordD4;
+  uintptr_t geno_word = 0;
+  for (uint32_t widx = 0; ; ++widx) {
+    if (widx >= sample_ctl2m1) {
+      if (widx > sample_ctl2m1) {
+        if (sample_ct % 2) {
+					if (!skipFn(rx)) {
+						memcpy(result_iter + dr, &(table_alias[(geno_word & 3) * 2]), 8);
+					}
+        }
+        return;
+      }
+      loop_len = ModNz(sample_ct, kBitsPerWordD2) / 2;
+    }
+    geno_word = genoarr[widx];
+    for (uint32_t uii = 0; uii != loop_len; ++uii) {
+      const uintptr_t cur_2geno = geno_word & 15;
+			if (!skipFn(rx)) {
+				memcpy(result_iter + dr, &(table_alias[cur_2geno * 2]), 8);
+				dr += 1;
+			}
+			rx += 1;
+			if (!skipFn(rx)) {
+				memcpy(result_iter + dr, &(table_alias[cur_2geno * 2])+1, 8);
+				dr += 1;
+			}
+			rx += 1;
+      geno_word >>= 4;
     }
   }
 }
 
 const double kGenoDoublePairs[32] ALIGNV16 = PAIR_TABLE16(0.0, 1.0, 2.0, NA_REAL);
 
-void Dosage16ToDoublesMinus9(const uintptr_t* genoarr, const uintptr_t* dosage_present, const uint16_t* dosage_main, uint32_t sample_ct, uint32_t dosage_ct, double* geno_double)
+void Dosage16ToDoubles(const uintptr_t* genoarr, const uintptr_t* dosage_present,
+											 const uint16_t* dosage_main, uint32_t sample_ct, uint32_t dosage_ct,
+											 std::function<bool(int)> skipFn,
+											 std::vector<int> &srcDestMap, double* geno_double)
 {
-	Dosage16ToDoubles(kGenoDoublePairs, genoarr, dosage_present, dosage_main, sample_ct, dosage_ct, geno_double);
+	const double* geno_double_pair_table = kGenoDoublePairs;
+	myGenoarrLookup16x8bx2(genoarr, geno_double_pair_table, sample_ct, skipFn, geno_double);
+  if (dosage_ct) {
+		// Not all data has poor QC.
+    const uint16_t* dosage_main_iter = dosage_main;
+    uintptr_t sample_uidx_base = 0;
+    uintptr_t cur_bits = dosage_present[0];
+    for (uint32_t dosage_idx = 0; dosage_idx != dosage_ct; ++dosage_idx) {
+      const uintptr_t sample_uidx = BitIter1(dosage_present, &sample_uidx_base, &cur_bits);
+			int dr = srcDestMap[sample_uidx];
+			if (dr >= 0) {
+				geno_double[dr] = S_CAST(double, *dosage_main_iter) * 0.00006103515625;
+			}
+			dosage_main_iter += 1;
+    }
+  }
 }
 
-int LoadDataPGENProvider::getNumVariants()
+int LoadDataPGENProvider2::getNumVariants()
 { return pgen_info->raw_variant_ct; }
 
-void LoadDataPGENProvider::loadRowImpl(int index)
+void LoadDataPGENProvider2::loadRowImpl(int index)
 {
 	if (columns.size() != 1) mxThrow("%s: pgen only has 1 column, not %d",
 					 name, int(columns.size()));
@@ -241,7 +279,7 @@ void LoadDataPGENProvider::loadRowImpl(int index)
 		PreinitPgfi(pgen_info.get());
 		pgen_info->vrtypes = 0;
 		uint32_t cur_variant_ct = 0xffffffffU;
-		uint32_t cur_sample_ct = rows;
+		uint32_t cur_sample_ct = srcRows;
 		PgenHeaderCtrl header_ctrl;
 		uintptr_t pgfi_alloc_cacheline_ct;
 		char errstr_buf[kPglErrstrBufBlen];
@@ -305,7 +343,7 @@ void LoadDataPGENProvider::loadRowImpl(int index)
 		loadCounter += 1;
 	}
 
-        if (1+index > int(pgen_info->raw_variant_ct)) {
+	if (1+index > int(pgen_info->raw_variant_ct)) {
 	  mxThrow("%s: out of data (record %d requested but only %d in file)",
 		  name, 1+index, int(pgen_info->raw_variant_ct));
 	}
@@ -318,40 +356,24 @@ void LoadDataPGENProvider::loadRowImpl(int index)
 					 pgen_dosage_present, pgen_dosage_main, &dosage_ct);
 		if (reterr != kPglRetSuccess)
 			mxThrow("%s: read_dosages(varient %d) error code %d", name, index, int(reterr));
-		Dosage16ToDoublesMinus9(pgen_genovec, pgen_dosage_present, pgen_dosage_main,
-					pgen_info->raw_sample_ct, dosage_ct, stripeData[0].realData);
+		Dosage16ToDoubles(pgen_genovec, pgen_dosage_present, pgen_dosage_main,
+											pgen_info->raw_sample_ct, dosage_ct, [&](int rx)->bool{ return skipRow(rx); },
+											srcDestMap, stripeData[0].realData);
 
 		double *rd = stripeData[0].realData;
 		int nrows = 0;
-		for (int rx=0; rx < rows; ++rx) {
+		for (int rx=0; rx < destRows; ++rx) {
 			if (!std::isfinite(rd[rx])) continue;
 			maf += rd[rx];
 			nrows += 1;
 		}
 		maf /= 2.0 * nrows;
 	} else {
-		PglErr reterr = PgrGet1(pgen_subset_include_vec, pgen_subset_cumulative_popcounts,
-					pgen_info->raw_sample_ct, index, 1, pgen_state.get(), pgen_genovec);
-		if (reterr != kPglRetSuccess)
-			mxThrow("%s: read(varient %d) error code %d", name, index, int(reterr));
-
-		auto &rc = (*rawCols)[ columns[0] ];
-		if (rc.levels.size() != 3) mxThrow("%s: pgen files contain data with 3 levels (not %d)",
-						   name, int(rc.levels.size()));
-		GenoarrToFactor(pgen_genovec, pgen_info->raw_sample_ct, stripeData[0].intData);
-
-		int *rd = stripeData[0].intData;
-		int nrows = 0;
-		for (int rx=0; rx < rows; ++rx) {
-			if (rd[rx] == NA_INTEGER) continue;
-			maf += rd[rx];
-			nrows += 1;
-		}
-		maf /= 2.0 * nrows;
+		stop("Treating genetic data as an ordinal factor is not implemented");
 	}
 
 	for (int cx=0; cx < int(columns.size()); ++cx) {
-		(*rawCols)[ columns[cx] ].ptr = stripeData[cx];
+		(*rawCols)[ columns[cx] ].setBorrow(stripeData[cx]);
 	}
 	if (checkpoint) {
 		auto &cv = *checkpointValues;
@@ -361,7 +383,7 @@ void LoadDataPGENProvider::loadRowImpl(int index)
 
 void setup2(AddLoadDataProviderType aldp)
 {
-	int sz = sizeof(LoadDataProviderBase);
-	aldp(OPENMX_LOAD_DATA_API_VERSION, sz, new LoadDataPGENProvider());
-	aldp(OPENMX_LOAD_DATA_API_VERSION, sz, new LoadDataBGENProvider());
+	int sz2 = sizeof(LoadDataProviderBase2);
+	aldp(OPENMX_LOAD_DATA_API_VERSION, sz2, new LoadDataPGENProvider2());
+	aldp(OPENMX_LOAD_DATA_API_VERSION, sz2, new LoadDataBGENProvider2());
 }
